/******************************************************************************/
#include "tc1ctype.h"
#include "tc1cext.h"
/******************************************************************************/
/*                     Do processing for decoder 2.
*******************************************************************************/
int Dec2_Process()
{
 Init_Alpha_Beta_Dec2();
 Compute_Alpha_Beta_Dec2();
 Compute_APPLes_Dec2();
 return(0);
}
/******************************************************************************/
/* Initialize alpha[0][state] and beta[FRAME_SIZE-ONE][state] for 2nd decoder.
*******************************************************************************/
int Init_Alpha_Beta_Dec2()
{
 int state;
 for(state=ZERO;state<NUM_ST_STATES;state++)
  Get_Init_Alpha_Beta_2(state);
 return(0);
}
/******************************************************************************/
/* Get the initial value of alpha[0][state] and beta[FRAME_SIZE-ONE][state]
   for a given state for 2nd decoder.
*******************************************************************************/
int Get_Init_Alpha_Beta_2(state)
int state;
{
 int input,prv_st,iv_inxa,iv_inxb,gia_inx,gea_inx,gib_inx,geb_inx,prv_ip_bit;
 double gea,geb,gia,gib,proba,probb;
 Alpha[PRED_MEM_SIZE][state]=Beta[FRAME_SIZE-ONE][state]=0.0;
 iv_inxa=De_Iv_Map[PRED_MEM_SIZE];
 iv_inxb=De_Iv_Map[FRAME_SIZE-ONE];
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=STrel_PS[state][input];
  prv_ip_bit=STrel_Ip[state][input];
  gia_inx=STrel_G_I[prv_st][prv_ip_bit];
  gea_inx=STrel_G_E[prv_st][prv_ip_bit];
  gib_inx=STrel_G_I[state][input];
  geb_inx=STrel_G_E[state][input];
  gia=Gamma_I_D2[PRED_MEM_SIZE][gia_inx];
  gea=Gamma_E_P2[PRED_MEM_SIZE][gea_inx];
  gib=Gamma_I_D2[FRAME_SIZE-ONE][gib_inx];
  geb=Gamma_E_P2[FRAME_SIZE-ONE][geb_inx];
  proba=Prob[iv_inxa][prv_ip_bit];
  probb=Prob[iv_inxb][input];
  Alpha[PRED_MEM_SIZE][state]=Alpha[PRED_MEM_SIZE][state]+gea*gia*proba;
  Beta[FRAME_SIZE-ONE][state]=Beta[FRAME_SIZE-ONE][state]+geb*gib*probb;
 }
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec2()
{
 int time,state;
 for(time=PRED_MEM_SIZE+ONE;time<(FRAME_SIZE-ONE);time++)
 {
  for(state=ZERO;state<NUM_ST_STATES;state++)
   Get_New_Alpha_Beta_Dec2(time,state);
  Normalize_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec2(time,state)
int time,state;
{
 int input,prv_st,nxt_st,iv_inxa,iv_inxb,gia_inx,gea_inx,gib_inx,geb_inx;
 int prv_ip_bit;
 double gea,geb,gia,gib,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 iv_inxa=De_Iv_Map[time];
 iv_inxb=De_Iv_Map[FRAME_SIZE-ONE-time+PRED_MEM_SIZE];
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=STrel_PS[state][input];
  prv_ip_bit=STrel_Ip[state][input];
  nxt_st=STrel_NS[state][input];
  gia_inx=STrel_G_I[prv_st][prv_ip_bit];
  gea_inx=STrel_G_E[prv_st][prv_ip_bit];
  gib_inx=STrel_G_I[state][input];
  geb_inx=STrel_G_E[state][input];
  gia=Gamma_I_D2[time][gia_inx];
  gea=Gamma_E_P2[time][gea_inx];
  gib=Gamma_I_D2[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][gib_inx];
  geb=Gamma_E_P2[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][geb_inx];
  proba=Prob[iv_inxa][prv_ip_bit];
  probb=Prob[iv_inxb][input];
  prv_alpha=Alpha[time-ONE][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+PRED_MEM_SIZE+ONE][nxt_st];
  alpha=alpha+gea*gia*proba*prv_alpha;
  beta=beta+geb*gib*probb*nxt_beta;
 }
 Alpha[time][state]=alpha;
 Beta[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec2()
{
 int time;
 double app[NUM_INPUTS];
 for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec2(time,app);
  Get_Prob_Dec2(time,app);
 }
 return(0);
}
/******************************************************************************/
/* Get a posteriori probability due to +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Dec2(time,app)
int time;
double app[];
{
 int state,input,nxt_state,ge_inx;
 double ge,prv_alpha,nxt_beta;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=STrel_NS[state][input];
   ge_inx=STrel_G_E[state][input];
   ge=Gamma_E_P2[time][ge_inx];
   prv_alpha=nxt_beta=1.0;
   if(time > PRED_MEM_SIZE)
    prv_alpha=Alpha[time-ONE][state];
   if(time < (FRAME_SIZE-ONE))
    nxt_beta=Beta[time+ONE][nxt_state];
   app[input]=app[input]+ge*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*          Get the normalized probabilities in deinterleaved format.
*******************************************************************************/
int Get_Prob_Dec2(time,app)
int time;
double app[];
{
 int input,iv_inx;
 double sum;
 iv_inx=De_Iv_Map[time];
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Prob[iv_inx][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
