/******************************************************************************/
#include "tc1ctype.h"
#include "tc1cext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Add_Noise();
  if(Frame_Cnt > ZERO)
   Receive();
 }
 return(0);
}
/******************************************************************************/
/*                                Add noise.
*******************************************************************************/
int Add_Noise()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<FRAME_SIZE;cnt++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  N2_Val[URADIUS]=erand48(N3_Store);
  N2_Val[UTHETA]=erand48(N4_Store);
  My_Gauss(VARIANCE,N1_Val);
  My_Gauss(VARIANCE,N2_Val);
  if(WHITE)
   Add_White_Noise(cnt);
  else if(FILTER == 1)
   Add_Filter1_Coloured_Noise(cnt);
  else if(FILTER == 2)
   Add_Filter2_Coloured_Noise(cnt);
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_White_Noise(time)
int time;
{
 Sym0[time]=Sym0[time]+N1_Val[GRAND_0];
 Sym1[time]=Sym1[time]+N1_Val[GRAND_1];
 Sym2[time]=Sym2[time]+N2_Val[GRAND_0];
 Sym3[time]=Sym3[time]+N2_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                     Get coloured noise from type 1 filter.
*******************************************************************************/
int Add_Filter1_Coloured_Noise(time)
int time;
{
 int cnt;
 Noise_0=Ip_Coeff*N1_Val[GRAND_0];
 Noise_1=Ip_Coeff*N1_Val[GRAND_1];
 Noise_2=Ip_Coeff*N2_Val[GRAND_0];
 Noise_3=Ip_Coeff*N2_Val[GRAND_1];
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
 {
  Noise_0=Noise_0-Coeff_Array[cnt]*N0_Reg[cnt];
  Noise_1=Noise_1-Coeff_Array[cnt]*N1_Reg[cnt];
  Noise_2=Noise_2-Coeff_Array[cnt]*N2_Reg[cnt];
  Noise_3=Noise_3-Coeff_Array[cnt]*N3_Reg[cnt];
 }
 Sym0[time]=Sym0[time]+Noise_0;
 Sym1[time]=Sym1[time]+Noise_1;
 Sym2[time]=Sym2[time]+Noise_2;
 Sym3[time]=Sym3[time]+Noise_3;
 Shift_In_New_Noise_Filter1();
 return(0);
}
/******************************************************************************/
/*             Shift in the new noise samples into type 1 filter.
*******************************************************************************/
int Shift_In_New_Noise_Filter1()
{
 int cnt;
 for(cnt=PRED_MEM_SIZE-ONE;cnt>ZERO;cnt--)
 {
  N0_Reg[cnt]=N0_Reg[cnt-ONE];
  N1_Reg[cnt]=N1_Reg[cnt-ONE];
  N2_Reg[cnt]=N2_Reg[cnt-ONE];
  N3_Reg[cnt]=N3_Reg[cnt-ONE];
 }
 N0_Reg[ZERO]=Noise_0;
 N1_Reg[ZERO]=Noise_1;
 N2_Reg[ZERO]=Noise_2;
 N3_Reg[ZERO]=Noise_3;
 return(0);
}
/******************************************************************************/
/*                     Get coloured noise from type 2 filter.
*******************************************************************************/
int Add_Filter2_Coloured_Noise(time)
int time;
{
 int cnt;
 Noise_0=Noise_1=Noise_2=Noise_3=0.0;
 Shift_In_New_Noise_Filter2();
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
 {
  Noise_0=Noise_0+F0_Reg[cnt]*Filter_Array[cnt];
  Noise_1=Noise_1+F1_Reg[cnt]*Filter_Array[cnt];
  Noise_2=Noise_2+F2_Reg[cnt]*Filter_Array[cnt];
  Noise_3=Noise_3+F3_Reg[cnt]*Filter_Array[cnt];
 }
 Sym0[time]=Sym0[time]+Noise_0;
 Sym1[time]=Sym1[time]+Noise_1;
 Sym2[time]=Sym2[time]+Noise_2;
 Sym3[time]=Sym3[time]+Noise_3;
 return(0);
}
/******************************************************************************/
/*              Shift in the new noise samples into type 2 filter.
*******************************************************************************/
int Shift_In_New_Noise_Filter2()
{
 int cnt;
 for(cnt=FILTER_SIZE-ONE;cnt>ZERO;cnt--)
 {
  F0_Reg[cnt]=F0_Reg[cnt-ONE];
  F1_Reg[cnt]=F1_Reg[cnt-ONE];
  F2_Reg[cnt]=F2_Reg[cnt-ONE];
  F3_Reg[cnt]=F3_Reg[cnt-ONE];
 }
 F0_Reg[ZERO]=N1_Val[GRAND_0];
 F1_Reg[ZERO]=N1_Val[GRAND_1];
 F2_Reg[ZERO]=N2_Val[GRAND_0];
 F3_Reg[ZERO]=N2_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Gamma_I_E();
 Init_App();
 Bcjr();
 Get_Bit_Errors_Dec1();
 Symbol_by_Symbol_Detection();
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_I_E()
{
 Init_Gamma_I_E_Dec1();
 Init_Gamma_I_E_Dec2();
 return(0);
}
/******************************************************************************/
/* Initialize the intrinsic and extrinsic values of gamma for decoder 1. Most
 * recent bit is at location ZERO.
*******************************************************************************/
int Init_Gamma_I_E_Dec1()
{
 int time,inx,array[PRED_MEM_SIZE+ONE];
 for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
 {
  for(inx=ZERO;inx<MOD1_NUM_INPUTS;inx++)
  {
   Decimal_to_Binary(inx,array,PRED_MEM_SIZE+ONE);
   Compute_Gamma_E_P1(inx,time,array);
   Compute_Gamma_I_D1(inx,time,array);
  }
 }
 return(0);
}
/******************************************************************************/
/*     Compute gamma from parity bit (extrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_E_P1(inx,time,array)
int inx,time,array[];
{
 int cnt,bit;
 double sym,x;
 x=0.0;
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  bit=array[cnt];
  sym=Bit_Map[bit];
  x=x+Coeff_Array[cnt-ONE]*(sym-Sym1[time-cnt]);
 }
 bit=array[ZERO];
 sym=Bit_Map[bit];
 x=x+sym-Sym1[time];
 x= -x*x*Inv_Twice_Pred_Var;
 Gamma_E_P1[time][inx]=exp(x);
// printf("time:%d bit:%d x:%lf ge1:%lf\n",time,inx,x,Gamma_E_P1[time][inx]);
 return(0);
}
/******************************************************************************/
/*      Compute gamma from data bit (intrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_I_D1(inx,time,array)
int inx,time,array[];
{
 int cnt,bit;
 double sym,x;
 x=0.0;
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  bit=array[cnt];
  sym=Bit_Map[bit];
  x=x+Coeff_Array[cnt-ONE]*(sym-Sym0[time-cnt]);
 }
 bit=array[ZERO];
 sym=Bit_Map[bit];
 x=x+sym-Sym0[time];
 x= -x*x*Inv_Twice_Pred_Var;
 Gamma_I_D1[time][inx]=exp(x);
// printf("time:%d bit:%d x:%lf gi1:%lf\n",time,inx,x,Gamma_I_D1[time][inx]);
 return(0);
}
/******************************************************************************/
/* Initialize the extrinsic values of gamma for decoder 2. The most recent
 * bit is at location 0.
*******************************************************************************/
int Init_Gamma_I_E_Dec2()
{
 int time,inx,array[PRED_MEM_SIZE+ONE];
 for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
 {
  for(inx=ZERO;inx<MOD1_NUM_INPUTS;inx++)
  {
   Decimal_to_Binary(inx,array,PRED_MEM_SIZE+ONE);
   Compute_Gamma_E_P2(inx,time,array);
   Compute_Gamma_I_D2(inx,time,array);
  }
 }
 return(0);
}
/******************************************************************************/
/*     Compute gamma from parity bit (extrinsic information) for decoder 2.
*******************************************************************************/
int Compute_Gamma_E_P2(inx,time,array)
int inx,time,array[];
{
 int cnt,bit;
 double sym,x;
 x=0.0;
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  bit=array[cnt];
  sym=Bit_Map[bit];
  x=x+Coeff_Array[cnt-ONE]*(sym-Sym2[time-cnt]);
 }
 bit=array[ZERO];
 sym=Bit_Map[bit];
 x=x+sym-Sym2[time];
 x= -x*x*Inv_Twice_Pred_Var;
 Gamma_E_P2[time][inx]=exp(x);
 return(0);
}
/******************************************************************************/
/*      Compute gamma from data bit (intrinsic information) for decoder 2.
*******************************************************************************/
int Compute_Gamma_I_D2(inx,time,array)
int inx,time,array[];
{
 int cnt,bit;
 double sym,x;
 x=0.0;
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  bit=array[cnt];
  sym=Bit_Map[bit];
  x=x+Coeff_Array[cnt-ONE]*(sym-Sym3[time-cnt]);
 }
 bit=array[ZERO];
 sym=Bit_Map[bit];
 x=x+sym-Sym3[time];
 x= -x*x*Inv_Twice_Pred_Var;
 Gamma_I_D2[time][inx]=exp(x);
// printf("time:%d bit:%d x:%lf gi:%lf\n",time,bit,x,Gamma_I[time][bit]);
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec1_Process();
  if(Iter < (MAX_ITER-ONE))
   Dec2_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  else
   detected_bit=ONE;
  if(detected_bit != Bits[time])
  {
   Ber++;
// printf("time:%d prob 0:%lf prob 1:%lf\n",time,Prob[time][1],Prob[time][0]);
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Do symbol-by-symbol detection.
*******************************************************************************/
int Symbol_by_Symbol_Detection()
{
 int time,bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(Sym0[time] > 0.0)
   bit=ZERO;
  else
   bit=ONE;
  if(bit != Bits[time])
   SS_Detect_Error++;
 }
 return(0);
}
/******************************************************************************/
