/******************************************************************************/
#include "tc3ctype.h"
#include "tc3cext.h"
/******************************************************************************/
/*                   Initialize simulation variables.
*******************************************************************************/
int Sim_Init()
{
 STrel_Init();
 Noise_Gen_Init();
 Iv_Noise_Gen_Init();
 Coloured_Noise_Filter_Init();
 Levinson_Durbin();
 Inv_Pred_Var_Init();
 Ber_Init();
 return(0);
}
/******************************************************************************/
/*                 Initialize some super trellis parameters.
*******************************************************************************/
int STrel_Init()
{
 int cnt;
 Inx_To_VG_Tbl=ZERO;
 for(cnt=ZERO;cnt<VG_TBL_SIZE;cnt++)
  Valid_Gamma_Inx[cnt]= MINUSONE;
 return(0);
}
/******************************************************************************/
/*                     Initialize the noise generator.
*******************************************************************************/
int Noise_Gen_Init()
{
 int cnt;
 long int random(void);
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
 {
  N1_Store[cnt]=(unsigned short)random();
  N2_Store[cnt]=(unsigned short)random();
  N3_Store[cnt]=(unsigned short)random();
  N4_Store[cnt]=(unsigned short)random();
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the interleaver noise generator.
*******************************************************************************/
int Iv_Noise_Gen_Init()
{
 Iv_Noise_Store[0]=15131;
 Iv_Noise_Store[1]=13111;
 Iv_Noise_Store[2]=10591;
 return(0);
}
/******************************************************************************/
/*          Initialize the filters that generate coloured noise.
*******************************************************************************/
int Coloured_Noise_Filter_Init()
{
 Get_Noise_Filter();
 Normalize_Filter_Energy();
 Print_Norm_Filter();
 Get_Auto_Corr();
 return(0);
}
/******************************************************************************/
/*             Read the noise filter coefficients from a file.
*******************************************************************************/
int Get_Noise_Filter()
{
 int cnt;
 double re_tap,im_tap;
 FILE *fp;
 cnt=ZERO;
 fp=fopen("tc3cdata/filter.dat","r");
 if(fp == NULL)
  Error_Mes(14);
 while(fscanf(fp,"%lf %lf",&im_tap,&re_tap) != EOF)
 {
  if(cnt == FILTER_SIZE)
   Error_Mes(12);
  Re_Flt[cnt]=re_tap;
  Im_Flt[cnt]=im_tap;
  cnt++;
 }
 if(cnt != FILTER_SIZE)
  Error_Mes(13);
 fclose(fp);
 return(0);
}
/******************************************************************************/
/*                  Normalize the filter to unit energy.
*******************************************************************************/
int Normalize_Filter_Energy()
{
 int cnt;
 double energy;
 energy=0.0;
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
  energy=energy+Re_Flt[cnt]*Re_Flt[cnt]+Im_Flt[cnt]*Im_Flt[cnt];
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
 {
  Re_Flt[cnt]=Re_Flt[cnt]/sqrt(energy);
  Im_Flt[cnt]=Im_Flt[cnt]/sqrt(energy);
 }
 return(0);
}
/******************************************************************************/
/*                 Print the normalized filter coefficients.
*******************************************************************************/
int Print_Norm_Filter()
{
 int cnt;
 FILE *fp;
 fp=fopen("tc3cdata/normflt.dat","w");
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
 {
  printf("\t Inx:%d Im:%f Re:%lf\n",cnt,Im_Flt[cnt],Re_Flt[cnt]);
  fprintf(fp,"%lf %lf\n",Im_Flt[cnt],Re_Flt[cnt]);
 }
 printf("\n");
 fclose(fp);
 return(0);
}
/******************************************************************************/
/*                Get the autocorrelation of the noise filter.
*******************************************************************************/
int Get_Auto_Corr()
{
 int cnt,inx,size;
 double re_mf[FILTER_SIZE],im_mf[FILTER_SIZE];
 double re_op[2*FILTER_SIZE-ONE],im_op[2*FILTER_SIZE-ONE];
 size=FILTER_SIZE;
 Copy_Rev_Conjugate(Re_Flt,Im_Flt,re_mf,im_mf,size);
 Convolve(Re_Flt,Im_Flt,re_mf,im_mf,re_op,im_op,size,size);
 for(cnt=FILTER_SIZE-ONE;cnt<=FILTER_SIZE-ONE+PRED_MEM_SIZE;cnt++)
 {
  inx=cnt-FILTER_SIZE+ONE;
  Re_AC[inx]=VARIANCE*re_op[cnt];
  Im_AC[inx]=VARIANCE*im_op[cnt];
  printf("\t Inx:%d ReAC::%lf ImAC:%lf\n",inx,Re_AC[inx],Im_AC[inx]);
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*          Initialize the inverse of the prediction error variance.
*******************************************************************************/
int Inv_Pred_Var_Init()
{
 Inv_Twice_Pred_Var=1.0/(2.0*Pred_Var);
 printf("\t Inv. of twice the pred. error var:%lf\n\n",Inv_Twice_Pred_Var);
 return(0);
}
/******************************************************************************/
/*                       Initialize the bit error rate.
*******************************************************************************/
int Ber_Init()
{
 Ber=ZERO;
 return(0);
}
/******************************************************************************/
