/******************************************************************************/
#include "tc4ctype.h"
#include "tc4cext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Add_Noise();
  if(Frame_Cnt > ZERO)
   Receive();
 }
 return(0);
}
/******************************************************************************/
/*                                Add noise.
*******************************************************************************/
int Add_Noise()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<FRAME_SIZE;cnt++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  N2_Val[URADIUS]=erand48(N3_Store);
  N2_Val[UTHETA]=erand48(N4_Store);
  My_Gauss(VARIANCE,N1_Val);
  My_Gauss(VARIANCE,N2_Val);
  if(WHITE)
   Add_White_Noise(cnt);
  else
   Add_Coloured_Noise(cnt);
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_White_Noise(time)
int time;
{
 Re_Sym0[time]=Re_Sym0[time]+N1_Val[GRAND_0];
 Im_Sym0[time]=Im_Sym0[time]+N1_Val[GRAND_1];
 Re_Sym1[time]=Re_Sym1[time]+N2_Val[GRAND_0];
 Im_Sym1[time]=Im_Sym1[time]+N2_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                     Get coloured noise from type 2 filter.
*******************************************************************************/
int Add_Coloured_Noise(time)
int time;
{
 int cnt;
 Re_Noise_0=Im_Noise_0=Re_Noise_1=Im_Noise_1=0.0;
 Shift_New_Noise_Into_Filter();
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
 {
  Re_Noise_0=Re_Noise_0+Re_F0_Reg[cnt]*Re_Flt[cnt];
  Re_Noise_0=Re_Noise_0-Im_F0_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_0=Im_Noise_0+Re_F0_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_0=Im_Noise_0+Im_F0_Reg[cnt]*Re_Flt[cnt];
  Re_Noise_1=Re_Noise_1+Re_F1_Reg[cnt]*Re_Flt[cnt];
  Re_Noise_1=Re_Noise_1-Im_F1_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_1=Im_Noise_1+Re_F1_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_1=Im_Noise_1+Im_F1_Reg[cnt]*Re_Flt[cnt];
 }
 Re_Sym0[time]=Re_Sym0[time]+Re_Noise_0;
 Im_Sym0[time]=Im_Sym0[time]+Im_Noise_0;
 Re_Sym1[time]=Re_Sym1[time]+Re_Noise_1;
 Im_Sym1[time]=Im_Sym1[time]+Im_Noise_1;
 return(0);
}
/******************************************************************************/
/*              Shift in the new noise samples into type 2 filter.
*******************************************************************************/
int Shift_New_Noise_Into_Filter()
{
 int cnt;
 for(cnt=FILTER_SIZE-ONE;cnt>ZERO;cnt--)
 {
  Re_F0_Reg[cnt]=Re_F0_Reg[cnt-ONE];
  Im_F0_Reg[cnt]=Im_F0_Reg[cnt-ONE];
  Re_F1_Reg[cnt]=Re_F1_Reg[cnt-ONE];
  Im_F1_Reg[cnt]=Im_F1_Reg[cnt-ONE];
 }
 Re_F0_Reg[ZERO]=N1_Val[GRAND_0];
 Im_F0_Reg[ZERO]=N1_Val[GRAND_1];
 Re_F1_Reg[ZERO]=N2_Val[GRAND_0];
 Im_F1_Reg[ZERO]=N2_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Gamma();
 Init_App();
 Bcjr();
 Get_Bit_Errors_Dec1();
 return(0);
}
/******************************************************************************/
/* Initialize the intrinsic and extrinsic values of gamma for decoder 1. Most
 * recent bit is at location ZERO.
*******************************************************************************/
int Init_Gamma()
{
 int time,inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(inx=ZERO;inx<NUM_GAMMA_INX;inx++)
  {
   Compute_Gamma_Dec1(inx,time);
   Compute_Gamma_Dec2(inx,time);
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Compute gammas for decoder 1.
*******************************************************************************/
int Compute_Gamma_Dec1(inx,time)
int inx,time;
{
 int cnt,re_bit,im_bit;
 double re_err,im_err,x;
 re_bit=inx%NUM_INPUTS;
 im_bit=inx/NUM_INPUTS;
 re_err=Bit_Map[re_bit]-Re_Sym0[time];
 im_err=Bit_Map[im_bit]-Im_Sym0[time];
 x=re_err*re_err+im_err*im_err;
 x= -x*Inv_Twice_Var;
 Gamma_Dec1[time][inx]=exp(x);
// printf("time:%d inx:%d x:%lf g:%lf\n",time,inx,x,Gamma_Dec1[time][inx]);
 return(0);
}
/******************************************************************************/
/*                        Compute gammas for decoder 2.
*******************************************************************************/
int Compute_Gamma_Dec2(inx,time)
int inx,time;
{
 int cnt,re_bit,im_bit;
 double re_err,im_err,x;
 re_bit=inx%NUM_INPUTS;
 im_bit=inx/NUM_INPUTS;
 re_err=Bit_Map[re_bit]-Re_Sym1[time];
 im_err=Bit_Map[im_bit]-Im_Sym1[time];
 x=re_err*re_err+im_err*im_err;
 x= -x*Inv_Twice_Var;
 Gamma_Dec2[time][inx]=exp(x);
// printf("time:%d bit:%d x:%lf ge1:%lf\n",time,inx,x,Gamma_E_P1[time][inx]);
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec1_Process();
  if(Iter < (MAX_ITER-ONE))
   Dec2_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  else
   detected_bit=ONE;
  if(detected_bit != Bits[time])
  {
   Ber++;
// printf("time:%d prob 0:%lf prob 1:%lf\n",time,Prob[time][1],Prob[time][0]);
  }
 }
 return(0);
}
/******************************************************************************/
