/******************************************************************************/
#include "tc5ctype.h"
#include "tc5cext.h"
/******************************************************************************/
/*                     Do processing for decoder 1.
*******************************************************************************/
int Dec1_Process()
{
 Init_Alpha_Beta_Dec1();
 Compute_Alpha_Beta_Dec1();
 if(Iter < (MAX_ITER-ONE))
  Compute_APPLes_Dec1();
 else if(Iter == (MAX_ITER-ONE))
  Compute_Final_APPs_Dec1();
 return(0);
}
/******************************************************************************/
/* Initialize alpha[0][state] and beta[FRAME_SIZE-ONE][state] for 1st decoder.
*******************************************************************************/
int Init_Alpha_Beta_Dec1()
{
 int state;
 for(state=ZERO;state<NUM_ST_STATES;state++)
  Get_Init_Alpha_Beta_1(state);
 Normalize_Alpha_Beta(PRED_MEM_SIZE);
 return(0);
}
/******************************************************************************/
/* Get the initial value of Alpha[PRED_MEM_SIZE][state] and
 * Beta[FRAME_SIZE-ONE][state] for a given state for 1st decoder.
*******************************************************************************/
int Get_Init_Alpha_Beta_1(state)
int state;
{
 int input,prv_st,ga_inx,gb_inx,prv_ip_bit;
 double ga,gb,proba,probb;
 Alpha[PRED_MEM_SIZE+ONE][state]=Beta[FRAME_SIZE-ONE][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=STrel_PS[state][input];
  prv_ip_bit=STrel_Ip[state][input];
  ga_inx=STrel_Inx[prv_st][prv_ip_bit];
  gb_inx=STrel_Inx[state][input];
  ga=Gamma_Dec1[PRED_MEM_SIZE][ga_inx];
  gb=Gamma_Dec1[FRAME_SIZE-ONE][gb_inx];
  proba=Prob[PRED_MEM_SIZE][prv_ip_bit];
  probb=Prob[FRAME_SIZE-ONE][input];
  Alpha[PRED_MEM_SIZE+ONE][state]=Alpha[PRED_MEM_SIZE+ONE][state]+ga*proba;
  Beta[FRAME_SIZE-ONE][state]=Beta[FRAME_SIZE-ONE][state]+gb*probb;
 }
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec1()
{
 int time,state;
 for(time=PRED_MEM_SIZE+ONE;time<FRAME_SIZE-ONE;time++)
 {
  for(state=ZERO;state<NUM_ST_STATES;state++)
   Get_New_Alpha_Beta_Dec1(time,state);
  Normalize_Alpha_Beta(time);
//  if(time == PRINT_TIME)
//   Print_Alpha_Beta();
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec1(time,state)
int time,state;
{
 int input,prv_st,nxt_st,ga_inx,gb_inx,prv_ip_bit;
 double ga,gb,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=STrel_PS[state][input];
  prv_ip_bit=STrel_Ip[state][input];
  nxt_st=STrel_NS[state][input];
  ga_inx=STrel_Inx[prv_st][prv_ip_bit];
  gb_inx=STrel_Inx[state][input];
  ga=Gamma_Dec1[time][ga_inx];
  gb=Gamma_Dec1[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][gb_inx];
  proba=Prob[time][prv_ip_bit];
  probb=Prob[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][input];
  prv_alpha=Alpha[time][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+PRED_MEM_SIZE+ONE][nxt_st];
  alpha=alpha+ga*proba*prv_alpha;
  beta=beta+gb*probb*nxt_beta;
 }
 Alpha[time+ONE][state]=alpha;
 Beta[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec1()
{
 int time;
 double app[NUM_INPUTS];
 for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec1(time,app);
  Get_Prob_Dec1(time,app);
 }
 return(0);
}
/******************************************************************************/
/*              Get the a posteriori probability due to +1 and -1.
*******************************************************************************/
int Get_APPLe_Dec1(time,app)
int time;
double app[];
{
 int state,input,nxt_state,g_inx;
 double g,prv_alpha,nxt_beta;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=STrel_NS[state][input];
   g_inx=STrel_Inx[state][input];
   g=Gamma_Dec1[time][g_inx];
   prv_alpha=nxt_beta=1.0;
   if(time > PRED_MEM_SIZE)
    prv_alpha=Alpha[time][state];
   if(time < (FRAME_SIZE-ONE))
    nxt_beta=Beta[time+ONE][nxt_state];
   app[input]=app[input]+g*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Dec1(time,app)
int time;
double app[];
{
 int input;
 double sum;
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Prob[time][input]=app[input]/sum;
// printf("time:%d prob 0:%lf prob 1:%lf\n",time,Prob[time][0],Prob[time][1]);
 return(0);
}
/******************************************************************************/
