/******************************************************************************/
#include "tc5ctype.h"
#include "tc5cext.h"
/******************************************************************************/
/*                     Do processing for decoder 2.
*******************************************************************************/
int Dec2_Process()
{
 Init_Alpha_Beta_Dec2();
 Compute_Alpha_Beta_Dec2();
 Compute_APPLes_Dec2();
 return(0);
}
/******************************************************************************/
/* Initialize alpha[0][state] and beta[FRAME_SIZE-ONE][state] for 2nd decoder.
*******************************************************************************/
int Init_Alpha_Beta_Dec2()
{
 int state;
 for(state=ZERO;state<NUM_ST_STATES;state++)
  Get_Init_Alpha_Beta_2(state);
 Normalize_Alpha_Beta(PRED_MEM_SIZE);
 return(0);
}
/******************************************************************************/
/* Get the initial value of alpha[0][state] and beta[FRAME_SIZE-ONE][state]
   for a given state for 2nd decoder.
*******************************************************************************/
int Get_Init_Alpha_Beta_2(state)
int state;
{
 int input,prv_st,iv_inxa,iv_inxb,ga_inx,gb_inx,prv_ip_bit;
 double ga,gb,proba,probb;
 Alpha[PRED_MEM_SIZE+ONE][state]=Beta[FRAME_SIZE-ONE][state]=0.0;
 iv_inxa=De_Iv_Map[PRED_MEM_SIZE];
 iv_inxb=De_Iv_Map[FRAME_SIZE-ONE];
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=STrel_PS[state][input];
  prv_ip_bit=STrel_Ip[state][input];
  ga_inx=STrel_Inx[prv_st][prv_ip_bit];
  gb_inx=STrel_Inx[state][input];
  ga=Gamma_Dec2[PRED_MEM_SIZE][ga_inx];
  gb=Gamma_Dec2[FRAME_SIZE-ONE][gb_inx];
  proba=Prob[iv_inxa][prv_ip_bit];
  probb=Prob[iv_inxb][input];
  Alpha[PRED_MEM_SIZE+ONE][state]=Alpha[PRED_MEM_SIZE+ONE][state]+ga*proba;
  Beta[FRAME_SIZE-ONE][state]=Beta[FRAME_SIZE-ONE][state]+gb*probb;
 }
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec2()
{
 int time,state;
 for(time=PRED_MEM_SIZE+ONE;time<(FRAME_SIZE-ONE);time++)
 {
  for(state=ZERO;state<NUM_ST_STATES;state++)
   Get_New_Alpha_Beta_Dec2(time,state);
  Normalize_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec2(time,state)
int time,state;
{
 int input,prv_st,nxt_st,iv_inxa,iv_inxb,ga_inx,gb_inx,prv_ip_bit;
 double ga,gb,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 iv_inxa=De_Iv_Map[time];
 iv_inxb=De_Iv_Map[FRAME_SIZE-ONE-time+PRED_MEM_SIZE];
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=STrel_PS[state][input];
  prv_ip_bit=STrel_Ip[state][input];
  nxt_st=STrel_NS[state][input];
  ga_inx=STrel_Inx[prv_st][prv_ip_bit];
  gb_inx=STrel_Inx[state][input];
  ga=Gamma_Dec2[time][ga_inx];
  gb=Gamma_Dec2[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][gb_inx];
  proba=Prob[iv_inxa][prv_ip_bit];
  probb=Prob[iv_inxb][input];
  prv_alpha=Alpha[time][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+PRED_MEM_SIZE+ONE][nxt_st];
  alpha=alpha+ga*proba*prv_alpha;
  beta=beta+gb*probb*nxt_beta;
 }
 Alpha[time+ONE][state]=alpha;
 Beta[FRAME_SIZE-ONE-time+PRED_MEM_SIZE][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec2()
{
 int time;
 double app[NUM_INPUTS];
 for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec2(time,app);
  Get_Prob_Dec2(time,app);
 }
 return(0);
}
/******************************************************************************/
/* Get a posteriori probability due to +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Dec2(time,app)
int time;
double app[];
{
 int state,input,nxt_state,g_inx;
 double g,prv_alpha,nxt_beta;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=STrel_NS[state][input];
   g_inx=STrel_Inx[state][input];
   g=Gamma_Dec2[time][g_inx];
   prv_alpha=nxt_beta=1.0;
   if(time > PRED_MEM_SIZE)
    prv_alpha=Alpha[time][state];
   if(time < (FRAME_SIZE-ONE))
    nxt_beta=Beta[time+ONE][nxt_state];
   app[input]=app[input]+g*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*          Get the normalized probabilities in deinterleaved format.
*******************************************************************************/
int Get_Prob_Dec2(time,app)
int time;
double app[];
{
 int input,iv_inx;
 double sum;
 iv_inx=De_Iv_Map[time];
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Prob[iv_inx][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
