/******************************************************************************/
#include "tc5ctype.h"
#include "tc5cext.h"
/******************************************************************************/
/*                         Convert decimal to binary.
*******************************************************************************/
int Decimal_to_Binary(num,array,array_size)
int num,array[],array_size;
{
 int cnt;
 for(cnt=ZERO;cnt<array_size;cnt++)
  array[cnt]=ZERO;
 cnt=ZERO;
 while(num>ZERO)
 {
  if(cnt == array_size)
   Error_Mes(1);
  array[cnt]=num&1;
  num=num>>1;
  cnt++;
 }
 return(0);
}
/******************************************************************************/
/*               Convert the 1s to -1s suitable for xoring.
*******************************************************************************/
int Convert_for_Xor(array,size)
int array[],size;
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  if(array[cnt] == ONE)
   array[cnt]= MINUSONE;
 }
 return(0);
}
/******************************************************************************/
/*                Convert the state back to 1s and 0s format.
*******************************************************************************/
int Convert_Back_to_Binary(array,size)
int array[],size;
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  if(array[cnt] == MINUSONE)
   array[cnt]=ONE;
 }
 return(0);
}
/******************************************************************************/
/*                    Convert binary array to decimal.
*******************************************************************************/
int Binary_to_Decimal(array,size)
int array[],size;
{
 int cnt,pow,sum;
 sum=ZERO;
 pow=1;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  sum=sum+pow*array[cnt];
  pow=pow<<1;
 }
 return(sum);
}
/******************************************************************************/
/*                   Normalize the alpha and beta values.
*******************************************************************************/
int Normalize_Alpha_Beta(time)
int time;
{
 int state,beta_inx;
 double suma,sumb;
 suma=sumb=0.0;
 beta_inx=FRAME_SIZE-ONE-time+PRED_MEM_SIZE;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  suma=suma+Alpha[time+ONE][state];
  sumb=sumb+Beta[beta_inx][state];
 }
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  Alpha[time+ONE][state]=Alpha[time+ONE][state]/suma;
  Beta[beta_inx][state]=Beta[beta_inx][state]/sumb;
 }
 return(0);
}
/******************************************************************************/
/*                    Print the alpha and beta values.
*******************************************************************************/
int Print_Alpha_Beta()
{
 int cnt,beta_inx;
 beta_inx=FRAME_SIZE-ONE-PRINT_TIME+PRED_MEM_SIZE;
 for(cnt=ZERO;cnt<NUM_ST_STATES;cnt++)
 {
  printf("Iter:%d State:%d Alpha time:%d",Iter,cnt,PRINT_TIME);
  printf(" Alpha:%lf",Alpha[PRINT_TIME][cnt]);
  printf(" Beta time:%d Beta:%lf\n",beta_inx,Beta[beta_inx][cnt]);
 }
 return(0);
}
/******************************************************************************/
/* Complex convolution. One 'xxip1[]' is real and 'xxip2[]' are complex.
   Input arrays: 'xxip1[ip1size]' is first array.
                 'xxip2[ip2size]' is second array.
   Output array: 'xxop[]'.
*******************************************************************************/
int Convolve(reip1,imip1,reip2,imip2,reop,imop,ip1size,ip2size)
int ip1size,ip2size;
double reip1[],imip1[],reip2[],imip2[],reop[],imop[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  reop[opcnt]=imop[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    reop[opcnt]=reop[opcnt]+reip1[ipcnt]*reip2[opcnt-ipcnt];
    reop[opcnt]=reop[opcnt]-imip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+reip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+imip1[ipcnt]*reip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*             Copy a sequence in reverse complex conjugate order.
*******************************************************************************/
int Copy_Rev_Conjugate(re_orig,im_orig,re_copy,im_copy,size)
int size;
double re_orig[],im_orig[],re_copy[],im_copy[];
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  re_copy[cnt]=  re_orig[size-ONE-cnt];
  im_copy[cnt]= -im_orig[size-ONE-cnt];
 }
 return(0);
}
/******************************************************************************/
/*                 Right shift the contents of the registers.
*******************************************************************************/
int Right_Shift_Reg(re_reg,im_reg,size)
int size;
double re_reg[],im_reg[];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
 {
  re_reg[cnt]=re_reg[cnt-ONE];
  im_reg[cnt]=im_reg[cnt-ONE];
 }
 return(0);
}
/******************************************************************************/
/*       Convert uniform random variables to gaussian random variables.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r < FRAC_ZERO)
  Error_Mes(10);
 r=sqrt(r);
 theta=2.0*MY_PI*tnarry[UTHETA];
 tnarry[GRAND_0]=r*cos(theta);
 tnarry[GRAND_1]=r*sin(theta);
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  Error_Mes(5);
 return(0);
}
/******************************************************************************/
/*                             Error message.
*******************************************************************************/
int Error_Mes(num)
int num;
{
 fprintf(stdout,"\t Simulation error. Exiting with code %d.\n\n",num);
 exit(num);
 return(0);
}
/******************************************************************************/
