/******************************************************************************/
#include "tc8type.h"
#include "tc8ext.h"
/******************************************************************************/
/*                     Do processing for decoder 1.
*******************************************************************************/
int Dec1_Process()
{
 Init_Alpha_Beta_Dec1();
 Compute_Alpha_Beta_Dec1();
 if(Iter < (MAX_ITER-ONE))
  Compute_APPLes_Dec1();
 else if(Iter == (MAX_ITER-ONE))
  Compute_Final_APPs_Dec1();
 return(0);
}
/******************************************************************************/
/* Initialize alpha[0][state] and beta[FRAME_SIZE-ONE][state] for 1st decoder.
*******************************************************************************/
int Init_Alpha_Beta_Dec1()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Get_Init_Alpha_Beta_1(state);
 return(0);
}
/******************************************************************************/
/* Get the initial value of alpha[0][state] and beta[FRAME_SIZE-ONE][state]
   for a given state for 1st decoder.
*******************************************************************************/
int Get_Init_Alpha_Beta_1(state)
int state;
{
 int input,prv_st,opa,opb;
 double gea,geb,gia,gib,proba,probb;
 Alpha[ZERO][state]=Beta[FRAME_SIZE-ONE][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=Trel_PS[state][input];
  opa=Trel_Op[prv_st][input];
  opb=Trel_Op[state][input];                              /* Get encoded bit. */
  gea=Gamma_E_P1[ZERO][opa];                        /* Corresponds to parity. */
  geb=Gamma_E_P1[FRAME_SIZE-ONE][opb];              /* Corresponds to parity. */
  gia=Gamma_I[ZERO][input];                           /* Corresponds to data. */
  gib=Gamma_I[FRAME_SIZE-ONE][input];                 /* Corresponds to data. */
  proba=Prob[ZERO][input];
  probb=Prob[FRAME_SIZE-ONE][input];
  Alpha[ZERO][state]=Alpha[ZERO][state]+gea*gia*proba;
  Beta[FRAME_SIZE-ONE][state]=Beta[FRAME_SIZE-ONE][state]+geb*gib*probb;
 }
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec1()
{
 int time,state;
 for(time=ONE;time<FRAME_SIZE-ONE;time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Alpha_Beta_Dec1(time,state);
  Normalize_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec1(time,state)
int time,state;
{
 int input,prv_st,nxt_st,opa,opb;
 double gea,geb,gia,gib,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Trel_PS[state][input];
  nxt_st=Trel_NS[state][input];
  opa=Trel_Op[prv_st][input];                             /* Get encoded bit. */
  opb=Trel_Op[state][input];                              /* Get encoded bit. */
  gea=Gamma_E_P1[time][opa];                        /* Corresponds to parity. */
  geb=Gamma_E_P1[FRAME_SIZE-ONE-time][opb];         /* Corresponds to parity. */
  gia=Gamma_I[time][input];                           /* Corresponds to data. */
  gib=Gamma_I[FRAME_SIZE-ONE-time][input];            /* Corresponds to data. */
  proba=Prob[time][input];
  probb=Prob[FRAME_SIZE-ONE-time][input];
  prv_alpha=Alpha[time-ONE][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+ONE][nxt_st];
  alpha=alpha+gea*gia*proba*prv_alpha;
  beta=beta+geb*gib*probb*nxt_beta;
 }
 Alpha[time][state]=alpha;
 Beta[FRAME_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec1()
{
 int time;
 double app[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec1(time,app);
  Get_Prob_Dec1(time,app);
 }
 return(0);
}
/******************************************************************************/
/*  Get a posteriori probability due to +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Dec1(time,app)
int time;
double app[];
{
 int state,input,nxt_state,op;
 double ge,prv_alpha,nxt_beta;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];                          /* 0 maps to +1. */
   op=Trel_Op[state][input];
   ge=Gamma_E_P1[time][op];                         /* Corresponds to parity. */
   prv_alpha=nxt_beta=1.0;
   if(time > ZERO)
    prv_alpha=Alpha[time-ONE][state];
   if(time < (FRAME_SIZE-ONE))
    nxt_beta=Beta[time+ONE][nxt_state];
   app[input]=app[input]+ge*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Dec1(time,app)
int time;
double app[];
{
 int input;
 double sum;
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Prob[time][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
