/******************************************************************************/
#include "tc9type.h"
#include "tc9ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Add_Noise();
  Receive();
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<FRAME_SIZE;cnt++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  N2_Val[URADIUS]=erand48(N3_Store);
  N2_Val[UTHETA]=erand48(N4_Store);
  My_Gauss(VARIANCE,N1_Val);
  My_Gauss(VARIANCE,N2_Val);
  Sym0[cnt]=Sym0[cnt]+N1_Val[GRAND_0];
  Sym1[cnt]=Sym1[cnt]+N1_Val[GRAND_1];
  Sym2[cnt]=Sym2[cnt]+N2_Val[GRAND_0];
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Gamma_I_E();
 Init_App();
 Bcjr();
 Get_Bit_Errors_Dec1();
 if(Frame_Cnt == PRINT_FRAME)
  Print_Prob();
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_I_E()
{
 Init_Gamma_I_E_Dec1();
 Init_Gamma_I_E_Dec2();
 return(0);
}
/******************************************************************************/
/*    Initialize the intrinsic and extrinsic values of gamma for decoder 1.
*******************************************************************************/
int Init_Gamma_I_E_Dec1()
{
 int time,bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   Compute_Gamma_E_P1(bit,time);
   Compute_Gamma_I(bit,time);
  }
 }
 return(0);
}
/******************************************************************************/
/*     Compute gamma from parity bit (extrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_E_P1(bit,time)
int bit,time;
{
 double sym,x;
 if((time%2) == P1_EXISTS)
 {
  sym=Bit_Map[bit];
  x=sym-Sym1[time];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P1[time][bit]=exp(x);
 }
 else
  Gamma_E_P1[time][bit]=1.0;
// printf("time:%d bit:%d x:%lf ge1:%lf\n",time,bit,x,Gamma_E_P1[time][bit]);
 return(0);
}
/******************************************************************************/
/*      Compute gamma from data bit (intrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_I(bit,time)
int bit,time;
{
 double sym,x;
 sym=Bit_Map[bit];
 x=sym-Sym0[time];
 x= -x*x*Inv_Twice_Noise_Var;
 Gamma_I[time][bit]=exp(x);
// printf("time:%d bit:%d x:%lf gi:%lf\n",time,bit,x,Gamma_I[time][bit]);
 return(0);
}
/******************************************************************************/
/*    Initialize the intrinsic and extrinsic values of gamma for decoder 2.
*******************************************************************************/
int Init_Gamma_I_E_Dec2()
{
 int time,bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
   Compute_Gamma_E_P2(bit,time);
 }
 return(0);
}
/******************************************************************************/
/*     Compute gamma from parity bit (extrinsic information) for decoder 2.
*******************************************************************************/
int Compute_Gamma_E_P2(bit,time)
int bit,time;
{
 double sym,x;
 if((time%2) != P1_EXISTS)
 {
  sym=Bit_Map[bit];
  x=sym-Sym2[time];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P2[time][bit]=exp(x);
 }
 else
  Gamma_E_P2[time][bit]=1.0;
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 if(Frame_Cnt == PRINT_FRAME)
 {
  Prob[PRINT_TIME][0]=1.0;
  Prob[PRINT_TIME][1]=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec2_Process();
  Dec1_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit,loc_ber,act_bit;
 loc_ber=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  detected_bit=ONE;
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  act_bit=Bits[time];
  if(detected_bit != Bits[time])
  {
   Ber++;
   loc_ber++;
   if(Frame_Cnt == PRINT_FRAME)
   {
    printf("Frame:%d time:%d act bit:%d ",Frame_Cnt,time,act_bit);
    printf("Prob act bit:%lf ",Prob[time][act_bit]);
    printf("Prob det bit:%lf\n",Prob[time][detected_bit]);
   }
  }
 }
 printf("Frame:%d ber:%d\n",Frame_Cnt,loc_ber);
 return(0);
}
/******************************************************************************/
/*                       Print out the probabilities.
*******************************************************************************/
int Print_Prob()
{
 int time,act_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  act_bit=Bits[time];
  printf("Print prob Frame:%d time:%d tx bit:%d ",Frame_Cnt,time,act_bit);
  printf("Prob 0:%lf  ",Prob[time][0]);
  printf("Prob 1:%lf\n",Prob[time][1]);
 }
 return(0);
}
/******************************************************************************/
